/*
 * Modified_UDPApp
 * Reads analog ports 0 and 1, and sends a UDP packet with their value
 * Based on UDP endpoint app provided by AsyncLabs
 *
 * Written by Chris McNally - csm44@cornell.edu
 * Cornell University
 *
 * 
 * 
 *  Version History :
 *  Version        Author    Date       Comments
 *  1.0             CM        ?            Modified UDP endpoint example code to send packet at regular invervals.
 *  1.1             CM        ?            Added analog read portion, 
 *  2.0             CM      4/26/10        Added time stamping, sending time and ADC readings in packet. Also added max reading
 */

#include <WiShield.h>

#define WIRELESS_MODE_INFRA	1
#define WIRELESS_MODE_ADHOC	2

// Wireless configuration parameters ----------------------------------------
unsigned char local_ip[] = {192,168,1,100};	// IP address of WiShield
unsigned char gateway_ip[] = {192,168,1,1};	// router or gateway IP address
unsigned char subnet_mask[] = {255,255,255,0};	// subnet mask for the local network
const prog_char ssid[] PROGMEM = {/*removed*/};		// max 32 bytes

unsigned char security_type = 3;	// 0 - open; 1 - WEP; 2 - WPA; 3 - WPA2

// WPA/WPA2 passphrase
const prog_char security_passphrase[] PROGMEM = {/*removed*/};	// max 64 characters

// WEP 128-bit keys
// sample HEX keys
prog_uchar wep_keys[] PROGMEM = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d,	// Key 0
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00,	// Key 1
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00,	// Key 2
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00	// Key 3
				};

// setup the wireless mode
// infrastructure - connect to AP
// adhoc - connect to another WiFi device
unsigned char wireless_mode = WIRELESS_MODE_INFRA;

unsigned char ssid_len;
unsigned char security_passphrase_len;
//---------------------------------------------------------------------------
int maxReading1 = 0;
int maxReading2 = 0;
int tempReading1 = 0;
int tempReading2 = 0;
long readingTime1 = 0;
int packetSent = 0;
//long readingTime2 = 0;
//long tempTime = 0;

void setup()
{        
        WiFi.init();
       //Serial.begin(9600); 
}

void loop()
{
        //if we are on a new packet, assign a new time value
        if(packetSent)
        {
          readingTime1 = millis();
          //Serial.println(maxReading1);
          packetSent = 0;
          maxReading1 = 0;
          maxReading2 = 0;
        }
        int i;
        //Run for 20 samples to capture peak of voltage signal
        for(i=0;i<250;i++)
        {
          //Get ADC reading for both sensors
          tempReading1 = analogRead(0);
          tempReading2 = analogRead(1);
          //if either reading larger than max value seen since
          //last transmission, set as new max value and record
          //time
          if(tempReading1 > maxReading1){
              maxReading1 = tempReading1;
          }
          if(tempReading2 > maxReading2){
              maxReading2 = tempReading2;
          }
        }
        //run wifi routine
	WiFi.run();
}