function PowerMeter
%Powermeter - GUI interface for Wireless Power Meter project
%Created by Chris McNally - csm44
%Cornell University
%%%%%%%%%%%%%%%%%%%%%%%%%%%Revision history%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Date       Version       Notes
% 4-13-10     1.0          Initial version
% 5-11-10     2.0          Update graphical elements and new data packet format
% 5-13-10     2.1          Changed graph axis to look better, added sensor selector                    

%declare main figure window at position 300,300
%800x600 windows
meter = figure('Position',[ 5 5 1000 600]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%layout parameters%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Graph
%powerGraphPos1 = [70 20 400 200];
powerGraphPos2 = [70 40 850 260];
%powerGraphPos3 = [70 560 400 200];

%%%%%%%%%Parameters%%%%%%%%%%%%
%reference voltage on arduino
vRef = 3.3;
%Burden resistance
bR = 150;
%Estimated power factor
pFactor = .7;
%Estimated mains voltage
Vrms = 121;
%#of samples to display on graph
windowSize = 100;
samplesCollected = 0;
%stop flag
keepTruckin = 1;
%1 = Current
%2 = Apparent power
%3 = Real power
whatToGraph = 1;
offset = 450;
%current data
timeData1 = [];
currentData1 = [];
%timeData2 = [];
currentData2 = [];
avg = '';

graphData = [];
graphTime = [];

receiveTime = [];
plotTime = [];
overallTime = [];
receiveStartTime = [];
plotStartTime = [];
overallStartTime = [];

%%%%%%%%Button positions%%%%%%%%%
startButtonPos = [660-offset 560 60 30];
stopButtonPos = [760-offset 560 60 30];
setRPos = [790-offset 500+20 60 20];
setVrefPos = [790-offset 450+20 60 20];
setPfPos = [790-offset 400+20 60 20];

%%%%Parameter UI positions%%%%%
rTextBoxPos = [690-offset 500+20 60 20];
rLabel = [540-offset 500+20 100 20];
vRefTextBoxPos = [690-offset 450+20 60 20];
vRefLabel = [540-offset 450+20 100 20];
pfTextBoxPos = [690-offset 400+20 60 20];
pfLabel = [540-offset 400+20 100 20];
avgTextBoxPos = [];
avgLabel = [];
graphSelectPos = [690-offset 350+20 100 20];
graphSelectLabelPos = [540-offset 350+20 100 20];
sensorSelectPos = [690-offset 300+20 100 20];
sensorSelectLabelPos = [540-offset 300+20 100 20];
%%%%%%%%%%%%%%%%%%%%%%%%%%%Object declarations%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%Graphs%%%%%%%%%
%powerGraph1 = axes('Parent',meter,'Units','pixels','Position',powerGraphPos1);
%title('Sensor 1');
powerGraph2 = axes('Parent',meter,'Units','pixels','Position',powerGraphPos2);
title('Collected Data');
%powerGraph3 = axes('Parent',meter,'Units','pixels','Position',powerGraphPos3);
%title('Total');

%%%%%%%%%%Buttons%%%%%%%%%%%
startButton = uicontrol(meter,'Position',startButtonPos,...
                'String','Start',...
                'Callback',@collectData);
stopButton = uicontrol(meter,'Position',stopButtonPos,...
                'String','Stop',...
                'Callback',@stopCollectData);
            
setRButton = uicontrol(meter,'Position',setRPos,...
                'String','Set',...
                'Callback',@setR);
            
setVButton = uicontrol(meter,'Position',setVrefPos,...
                'String','Set',...
                'Callback',@setV);
            
setPFButton = uicontrol(meter,'Position',setPfPos,...
                'String','Set',...
                'Callback',@setPF);
%%%%%%%%Text fields%%%%%%%%
%%%User definable%%%%
burdenLabel = uicontrol(meter,'Style','text',...
                'String','Burden (Ohms)',...
                'Position',rLabel);

burdenR = uicontrol(meter,'Style','edit',...
                'String','150',...
                'Max',1,'Min',0,...
                'Position',rTextBoxPos);
            
voltageLabel = uicontrol(meter,'Style','text',...
                'String','Vref (V)',...
                'Position',vRefLabel);
            
voltageReference = uicontrol(meter,'Style','edit',...
                'String','3.3',...
                'Max',1,'Min',0,...
                'Position',vRefTextBoxPos);

pfLabel = uicontrol(meter,'Style','text',...
                'String','Est power factor',...
                'Position',pfLabel);
            
powerFactor = uicontrol(meter,'Style','edit',...
                'String','.7',...
                'Max',1,'Min',0,...
                'Position',pfTextBoxPos);
            
graphSelectLabel = uicontrol(meter,'Style','text',...
                'String','Graph value',...
                'Position',graphSelectLabelPos);
            
            
graphSelect = uicontrol(meter,'Style','popupmenu',...
                'String',{'Current','Est App Pow','Est Real Pow'},...
                'Value',1,'Position',graphSelectPos);

sensorSelectLabel = uicontrol(meter,'Style','text',...
                'String','Sensor Options',...
                'Position',sensorSelectLabelPos);
            
sensorSelect = uicontrol(meter,'Style','popupmenu',...
                'String',{'Sensor 0+1','Sensor 0','Sensor 1'},...
                'Value',1,'Position',sensorSelectPos);
%%%Outputs%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %receives UDP packets from Arduino
%     %function data = receiveData()
%         
%     end
% 
%     %grabs new data and graphs data
%     function updateGraph()
%         
%     end

    %loops, checks whether to collect data or not
    function collectData(hObject,eventdata)
        keepTruckin = 1;
        %grab new packet
        import java.io.*
        import java.net.DatagramSocket
        import java.net.DatagramPacket
        import java.net.InetAddress
        %receieve port, set on Arduino. Default is 12344.
        port = 12344;
        socket = DatagramSocket(port);
        %timeout if no packets are received (milliseconds)
        timeout = 1000;
        %estimated length of incoming packet
        packetLength = 100;
        socket.setSoTimeout(timeout);
        firstRun = 1;
        while(keepTruckin)
            receiveStartTime = tic;
            overallStartTime = tic;
            try                
                %grab the packet
                packet = DatagramPacket(zeros(1,packetLength,'int8'),packetLength);      
                %close socket
                socket.receive(packet);
                %extract data from packet
                mssg = packet.getData;
                mssg = mssg(1:packet.getLength);     
                newData = mssg;
            catch receiveError
                % Determine whether error occurred because of a timeout.
                if ~isempty(strfind(receiveError.message,'java.net.SocketTimeoutException'))
                    errorStr = sprintf('Failed to receive UDP packet; connection timed out.\n');
                else
                    errorStr = sprintf('Failed to receive UDP packet.\nJava error message follows:\n%s',receiveError.message);
                end % if        
                try
                    socket.close;            
                end % try
                error(errorStr);
            end % try
            receiveTime = [receiveTime toc(receiveStartTime)];
            %sort new data
            parsedNewData = cellstr(char(newData'));
            parsedNewData = parsedNewData{1};
            parsedNewData = sscanf(parsedNewData,'%d');
            %add new power data to data set
            timeData1 = [timeData1 ; parsedNewData(1)];
            currentData1 = [currentData1 ; parsedNewData(2) ./ (1023.*sqrt(2)*bR) .* vRef .* 3100];
            %timeData2 = [timeData2 ; parsedNewData(3)];
            currentData2 = [currentData2 ; parsedNewData(3)  ./ (1023.*sqrt(2)*bR) .* vRef .* 3100];
            whatToGraph = get(graphSelect,'Value');
            switch whatToGraph
                %%%%%%%%%Graphing current data%%%%%%%%
                case 1
                    %samplesCollected = samplesCollected + 1;
                    if(windowSize > samplesCollected)
                        graphData =  currentData1 + currentData2;
                        graphTime = timeData1;
                        noGraph = 1;
                    else
                        graphData = currentData1(samplesCollected+1-windowSize:end) + ...
                            currentData2(samplesCollected+1-windowSize:end);
                        graphTime = timeData1(samplesCollected+1-windowSize:end);
                        noGraph = 0;
                    end
                    yString = 'RMS current (amps)';
                    xMax = 80;
                %%%%%Graphing apparent power%%%%%%%%
                case 2
                %Apparent power = Vrms * Irms
                    if(windowSize > samplesCollected)
                        graphData =  (currentData1 + currentData2) .* Vrms;
                        graphTime = timeData1;
                    else
                        graphData = (currentData1(samplesCollected+1-windowSize:end,:) + ...
                            currentData2(samplesCollected+1-windowSize:end,:)) .* Vrms;
                        graphTime = timeData1(samplesCollected+1-windowSize:end);
                    end
                    yString = 'Apparent power (VA)';
                    xMax = max(graphData);
                %%%%%%graphing real power%%%%%%%
                case 3
                    %Real power = Apparent power * power factor
                    if(windowSize > samplesCollected)
                        graphData =  (currentData1 + currentData2) .* Vrms .*pFactor;
                        graphTime = timeData1;
                    else
                        graphData = (currentData1(samplesCollected+1-windowSize:end,:) + ...
                        currentData2(samplesCollected+1-windowSize:end,:)).* Vrms .*pFactor;
                        graphTime = timeData1(samplesCollected+1-windowSize:end);
                    end
                    yString = 'Real power (W)';
                    xMax = max(graphData);
            end
            plotStartTime =  tic;
            samplesCollected = samplesCollected + 1;
            %cla;
            if(firstRun)
                noGraph = 0;
                firstRun = 0;
                plotHandle = plot(powerGraph2,graphTime,graphData); 
            else
                if(noGraph)
                else
                    set(plotHandle,'Xdata',graphTime);
                    set(plotHandle,'Ydata',graphData);
                end 
            end
            title('Collected Data');
            xlabel('time from power up (ms)');
            ylabel(yString);
            axis(powerGraph2,[min(graphTime) max(graphTime)+1 0 xMax+1]);
            axis manual;
            drawnow;
            plotTime = [plotTime toc(plotStartTime)];
            overallTime = [overallTime toc(overallStartTime)];
        end
        socket.close;
    end

    %sets stop flag
    function stopCollectData(hObject,eventdata)
        %grab current time
        c = datestr(now, 'mmm dd, yyyy HH MM SS'); 
        %save workspace
        save(c);
        keepTruckin = 0;
        samplesCollected = 0;
        currentData1 = [];
        currentData2 = [];
        timeData1 = [];
        graphData = [];
        graphTime = [];
        cla;
    end

    function setR(hObject,eventdata)
        bR = str2num(get(burdenR,'String'));
    end

    function setV(hObject,eventdata)
        vRef = str2num(get(voltageReference,'String'));
    end

    function setPF(hObject,eventdata)
        pFactor = str2num(get(powerFactor,'String'));
    end

end