
/******************************************************************************

  Filename:		udpapp.c
  Description:	UDP app for the WiShield 1.0

 ******************************************************************************
000
  TCP/IP stack and driver for the WiShield 1.0 wireless devices

  Copyright(c) 2009 Async Labs Inc. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Contact Information:
  <asynclabs@asynclabs.com>
  
  Modified by Chris McNally
  csm44@cornell.edu
  Cornell University

   Author               Date                Comment
  ---------------------------------------------------------------
   AsyncLabs	      07/11/2009       Initial version
   Chris McNally      3/25/2010        Modified to just send data, do ADC measurements
   Chris McNally      4/25/2010        Removed unused parts of original code, added external declarations 
 *****************************************************************************/

#include "uip.h"
#include <string.h>
#include "udpapp.h"
#include "config.h"


#define STATE_INIT				0
#define STATE_LISTENING         1
#define STATE_HELLO_RECEIVED	2
#define STATE_NAME_RECEIVED		3

//ADC measurements (range 0-1023)
extern int maxReading1;
extern int maxReading2;
extern long readingTime1;
//extern long readingTime2;
extern int packetSent;

static struct udpapp_state s;

void dummy_app_appcall(void)
{
}

void udpapp_init(void)
{
	uip_ipaddr_t addr;
	struct uip_udp_conn *c;
        //IP address to send packets to
	uip_ipaddr(&addr, 192,168,1,2);
        //create UDP connection to address at port given
	c = uip_udp_new(&addr, HTONS(12344));
	if(c != NULL) {
		uip_udp_bind(c, HTONS(12344));
	}
	s.state = STATE_INIT;
	PT_INIT(&s.pt);
}

static PT_THREAD(handle_connection(void))
{
	//mark beginning of protothreat
	PT_BEGIN(&s.pt);
	//translate data into string
	char data[100];
	sprintf(data,"%lu %u %u",readingTime1,maxReading1,maxReading2);
	//give string over to uIP, send packet
	uip_send(data,strlen(data));
        //reset max readings for next sampling period
        packetSent = 1;
        //mark end of protothread
	PT_END(&s.pt);
}

void udpapp_appcall(void)
{	
        handle_connection();
}
